
import tw2.jquery.base as twjq_c
import defaults

jqgrid_css = twjq_c.jQueryPluginCSSLink(
    name=defaults._jqgrid_name_,
    version = defaults._jqgrid_version_,
    basename = defaults._jqgrid_css_basename_,
    modname = 'tw2.jqplugins.jqgrid',
)
word_wrap_css = twjq_c.jQueryPluginCSSLink(
    name=defaults._jqgrid_name_,
    version = defaults._jqgrid_version_,
    basename = defaults._jqgrid_css_basename_,
    modname = 'tw2.jqplugins.jqgrid',
    variant='word_wrap',
)

jqgrid_locale = twjq_c.jQueryPluginJSLink(
    name=defaults._jqgrid_name_,
    basename='grid.locale-%s' % defaults._jqgrid_locale_,
    subdir='js/i18n',
    version=defaults._jqgrid_version_,
    modname='tw2.jqplugins.jqgrid',
)

jqgrid_js = twjq_c.jQueryPluginJSLink(
    name=defaults._jqgrid_name_,
    version=defaults._jqgrid_version_,
    variant='min',
    modname='tw2.jqplugins.jqgrid',
)

__all__ = ['jqgrid_js', 'jqgrid_locale', 'jqgrid_css']



import transaction
from sqlalchemy import (
    Column, Integer, Unicode,
    MetaData, Table, ForeignKey,
)
from sqlalchemy.orm import relation, backref
from sqlalchemy.ext.declarative import declarative_base

import tw2.sqla as tws
    
import random


session = tws.transactional_session()
Base = declarative_base(metadata=MetaData('sqlite:///%s.db' % __name__))
Base.query = session.query_property()

friends_mapping = Table(
    'persons_friends_mapping', Base.metadata,
    Column('friender_id', Integer,
           ForeignKey('persons.id'), primary_key=True),
    Column('friendee_id', Integer,
           ForeignKey('persons.id'), primary_key=True))

class Person(Base):
    __tablename__ = 'persons'
    id = Column(Integer, primary_key=True)
    first_name = Column(Unicode(255), nullable=False)
    last_name = Column(Unicode(255), nullable=False)
    some_attribute = Column(Unicode(255), nullable=False)

    # One-to-one
    pet = relation('Pet', backref='owner', uselist=False)
    # One-to-many
    children = relation('Child', backref='parent')
    # Many-to-one
    job_id = Column(Integer, ForeignKey('jobs.id'))

    def __unicode__(self):
        return "%s %s" % (self.first_name, self.last_name)

class Pet(Base):
    __tablename__ = 'pets'
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(255), nullable=False)
    variety = Column(Unicode(255), nullable=False)
    owner_id = Column(Integer, ForeignKey('persons.id'))

    def __unicode__(self):
        return self.name

class Child(Base):
    __tablename__ = 'children'
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(255), nullable=False)
    parent_id = Column(Integer, ForeignKey('persons.id'))

class Job(Base):
    __tablename__ = 'jobs'
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(255), nullable=False)
    persons = relation('Person', backref='job')

    def __unicode__(self):
        return self.name



Person.__mapper__.add_property('friends', relation(Person,
    primaryjoin=Person.id==friends_mapping.c.friendee_id,
    secondaryjoin=friends_mapping.c.friender_id==Person.id,
    secondary=friends_mapping,
    doc="List of this persons' friends!",
))


Base.metadata.create_all()

def populateDB(sess):
    if Person.query.count() > 0:
        print "Not populating DB.  Already stuff in there."
        return

    firsts = ["Sally", "Suzie", "Sandy",
              "John", "Jim", "Joseph"]
    lasts = ["Anderson", "Flanderson", "Johnson",
             "Frompson", "Qaddafi", "Mubarak", "Ben Ali"]

    for first in firsts:
        for last in lasts:
            p = Person(
                first_name=first, last_name=last,
                some_attribute="Fun fact #%i" % random.randint(0,255)
            )
            sess.add(p)

    job_names = ["Programmer", "Sysadmin", "Suit"]
    jobs = [Job(name=job_name) for job_name in job_names]
    [sess.add(job) for job in jobs]

    pet_names = ["Spot", "Mack", "Cracker", "Fluffy", "Alabaster",
                 "Slim Pickins", "Lil' bit", "Balthazaar", "Hadoop"]
    varieties = ["dog", "cat", "bird", "fish", "hermit crab", "lizard"]

    for person in Person.query.all():
        pet = Pet(name=pet_names[random.randint(0,len(pet_names)-1)],
                  variety=varieties[random.randint(0,len(varieties)-1)])
        sess.add(pet)
        person.pet = pet
        for i in range(0, random.randint(0, 4)):
            child = Child(name=firsts[random.randint(0, len(firsts)-1)])
            sess.add(child)
            person.children.append(child)
            person.job = jobs[random.randint(0, len(jobs)-1)]


    qaddafis = Person.query.filter_by(last_name='Qaddafi').all()
    mubaraks = Person.query.filter_by(last_name='Mubarak').all()
    benalis = Person.query.filter_by(last_name='Ben Ali').all()
    dictators = qaddafis + mubaraks + benalis

    print "populating dictators friends"
    for p1 in dictators:
        for p2 in dictators:
            if p1 == p2 or p1 in p2.friends:
                continue
            if random.random() > 0.25:
                p1.friends.append(p2)
                p2.friends.append(p1)

    print "populating everyone else's friends"
    for p1 in Person.query.all():
        for p2 in Person.query.all():
            if p1 == p2 or p1 in p2.friends:
                continue
            if random.random() > 0.95:
                p1.friends.append(p2)
                p2.friends.append(p1)

    print "done populating DB"

populateDB(session)
transaction.commit()



"""
Here you can create samples of your widgets by providing default parameters,
inserting them in a container widget, mixing them with other widgets, etc...
These samples will appear in the WidgetBrowser

See http://toscawidgets.org/documentation/WidgetBrowser for more information
"""
from tw2.jqplugins.jqgrid.widgets import jqGridWidget
from tw2.jqplugins.jqgrid.base import word_wrap_css
from random import random
from random import randint
r = lambda x : randint(0,x-1)

# No sample b/c its so difficult to find a datasource.

def generate_data(n):
    fnames = ['S.', 'J.']
    lnames = ['Flanderson', 'Anderson', 'Sanderson', 'Boot', 'Bean', 'Resig']
    intros = ['', 'On the', 'Regarding the', 'On', 'New Developments in',
              'A Survey of the',
             ]
    adjectives = ['Scientific', 'Schmientific', 'Cosmological', 'Astronomical',
                  'Computational', 'Fromputational', 'Mechanical', 'Chemical',
                  'Pythonic', 'Functional', 'Declarative', 'Quantum',
                  'Entanglement', 'Singular', 'Non-negative',
                 ]
    nouns = ['Problems', 'Solutions', 'Ins-and-outs', 'Nuances',
             'Transmogrification', 'Implementation',
            ]
    segues = ['of', 'underlying']
    journals = ['Review', 'Journal', 'Conference']
    for i in range(n):
        authors = ", ".join(["%s %s" % (fnames[r(len(fnames))],
                                        lnames[r(len(lnames))])
                             for j in range(randint(1,2))])
        title = "%s %s %s %s %s %s." %(
            intros[r(len(intros))],
            adjectives[r(len(adjectives))],
            nouns[r(len(nouns))],
            segues[r(len(segues))],
            adjectives[r(len(adjectives))],
            nouns[r(len(nouns))],
        )
        journal = "The %s of %s %s" % (
            journals[r(len(journals))],
            adjectives[r(len(adjectives))],
            nouns[r(len(nouns))],
        )
        published = randint(1955, 2010)
        yield {'authors':authors,
               'title':title,
               'journal':journal,
               'published_on':published}

class DemoJQGridWidget(jqGridWidget):

    def prepare(self):
        self.resources.append(word_wrap_css)
        super(DemoJQGridWidget, self).prepare()

    options = {
        'pager' : 'module-0-demo_pager',
        'caption' : 'All research publications',
        'data' : [row for row in generate_data(55)],
        'datatype' : 'local',
        'colNames':[ 'Authors', 'Title', 'Journal', 'Published' ],
        'colModel' : [
            {
                'name':'authors',
                'width':75,
                'align':'center',
            },{
                'name':'title',
            },{
                'name':'journal',
            },{
                'name':'published_on',
                'width':50,
                'align':'center'
            },
        ],
        'rowNum':15,
        'rowList':[15,30,50],
        'viewrecords':True,
        'imgpath': 'scripts/jqGrid/themes/green/images',
        'width': 900,
        'height': 'auto',
    }
    pager_options = { "search" : True, "refresh" : True, "add" : False, }
    prmSearch = {
        "sopt": ["cn", "bw"],
        "caption": "Search...",
        "multipleSearch": True,
    }
    custom_pager_buttons = [
        {
            "caption":"",
            "buttonicon":"ui-icon-newwin",
            'onClickButton': None,
            'position': "last",
            'title':"B1",
            'cursor':"pointer"
        },{
            'caption':"",
            'buttonicon':"ui-icon-cart",
            'onClickButton':None,
            'position': "first",
            'title':"B2",
            'cursor': "pointer"
        },
    ]






from tw2.core.resources import encoder
import tw2.core as twc

import tw2.jquery
import tw2.jquery.base as tw2_jq_c_b
import tw2.jqplugins.ui.base as tw2_jq_ui

import formencode.validators as fv
from tw2.jqplugins.jqgrid import base

_pager_defaults = {'enableSearch': True, 'enableClear': True, 'gridModel': True}

import formencode as fe
import formencode.validators as fv
# TODO -- how to make use of this?  a tw2 controller?
class jqGridFilterSchema(fe.Schema):
    allow_extra_fields=True

    page = fv.Int(if_empty=1, not_empty=False, if_missing=1)
    rows = fv.Int(if_empty=10, not_empty=False, if_missing=1)
    sidx = fv.String(if_empty=None, not_empty=False, if_missing=None)
    sord = fv.String(if_empty=None, not_empty=False, if_missing=None)
    _search = fv.StringBool(if_empty=False, not_empty=False, if_missing=False)
    searchField = fv.String(if_empty=None, not_empty=False, if_missing=None)
    searchString = fv.String(if_empty=None, not_empty=False, if_missing=None)
    searchOper = fv.String(if_empty=None, not_empty=False, if_missing=None)


class jqGridWidget(tw2_jq_ui.JQueryUIWidget):
    resources = [
        tw2.jquery.jquery_js,
        tw2_jq_ui.jquery_ui_js, tw2_jq_ui.jquery_ui_css,
        base.jqgrid_locale, base.jqgrid_js, base.jqgrid_css,
    ]
    template = "tw2.jqplugins.jqgrid.templates.jqgrid"
   
    options = twc.Param("Configuration options to pass to jqgrid", default={})
    pager_options = twc.Param("Configuration options for pager", default={})
    pager_id = twc.Variable("options['pager'] placeholder", default=None)

    prmFilter = twc.Param("params to pass to jqgrid filter toolbar", default={})
    _prmFilter = twc.Param("params to pass to jqgrid filter toolbar", default={})
    prmEdit   = twc.Param("params to pass to jqgrid pager [Edit]", default={})
    prmAdd    = twc.Param("params to pass to jqgrid pager [Add]", default={})
    prmDel    = twc.Param("params to pass to jqgrid pager [Del]", default={})
    prmSearch = twc.Param("params to pass to jqgrid pager [Search]", default={})
    prmView   = twc.Param("params to pass to jqgrid pager [View]", default={})
    # so this would be a list of dicts - and encoded as  [below]
    # self.custom_pager_buttons = map(encoder.encode,
    #                                 self.custom_pager_buttons)
    custom_pager_buttons = twc.Param("custom buttons to add to jqgrid pager",
                                     default=[])

    def prepare(self):
        if not self.options:
            raise ValueError, 'jqGridWidget must be supplied a dict of options'
        
        if (
            not 'url' in self.options and
            not 'data' in self.options and
            not 'datastr' in self.options):
            raise ValueError, "jqGridWidget must be supplied a " + \
                              "'url', 'data', or 'datastr' in options"

        self.pager_id = self.options.get('pager', None)
        super(jqGridWidget, self).prepare()
        self.pager_options = encoder.encode(self.pager_options)
        if self.prmFilter: self._prmFilter = True
        self.prmFilter  = encoder.encode(self.prmFilter)
        self.prmEdit    = encoder.encode(self.prmEdit)
        self.prmAdd     = encoder.encode(self.prmAdd)
        self.prmDel     = encoder.encode(self.prmDel)
        self.prmSearch  = encoder.encode(self.prmSearch)
        self.prmView    = encoder.encode(self.prmView)
        self.custom_pager_buttons = map(encoder.encode, self.custom_pager_buttons)



"""
Here you can create samples of your widgets by providing default parameters,
inserting them in a container widget, mixing them with other widgets, etc...
These samples will appear in the WidgetBrowser

See http://toscawidgets.org/documentation/WidgetBrowser for more information
"""
from tw2.jqplugins.jqgrid.widgets import SQLAjqGridWidget
from tw2.jqplugins.jqgrid.base import word_wrap_css

from tw2.jqplugins.jqgrid.samples import model

class DemoSQLAJQGridWidget(SQLAjqGridWidget):

    def prepare(self):
        self.resources.append(word_wrap_css)
        super(DemoSQLAJQGridWidget, self).prepare()

    entity = model.Person

    excluded_columns = ['id', 'job_id']

    options = {
        'url': '/db_jqgrid/',
        'rowNum':15,
        'rowList':[15,30,50],
        'viewrecords':True,
        'imgpath': 'scripts/jqGrid/themes/green/images',
        'width': 900,
        'height': 'auto',
    }

    prmFilter = {'stringResult': True, 'searchOnEnter': False}

    pager_options = { "search" : True, "refresh" : True, "add" : False, }


import tw2.core as twc
mw = twc.core.request_local()['middleware']
mw.controllers.register(DemoSQLAJQGridWidget, 'db_jqgrid')
